﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Linq.Expressions;
using System.Reflection;
using System.Text;
using Niwar.Framework.Data.PetaPoco.Internal;
using Niwar.Framework.ViewModel;

namespace Niwar.Framework.Data.PetaPoco
{
    /// <summary>
    /// PetaPoco扩展
    /// </summary>
    public static class PetaPocoExtensions 
    {
        /// <summary>
        /// 保存数据
        /// </summary>
        /// <param name="db"></param>
        /// <param name="poco">要保存的对象</param>
        /// <param name="updateNullFields">是否更新未赋值的字段</param>
        public static void Save(this Database db, object poco, bool updateNullFields)
        {
            var pd = PocoData.ForType(poco.GetType());
            string tableName=pd.TableInfo.TableName, primaryKeyName = pd.TableInfo.PrimaryKey;

            if (db.IsNew(primaryKeyName, poco))
            {
                var ExistColumns = poco.GetType().GetProperties().Where(p => p.GetCustomAttributes(typeof(ExistColumnAttribute),true).Length > 0).Select(p=>p.Name).ToArray();
                if (ExistColumns.Length > 0)
                {
                    Sql existSql = Sql.Builder.Select("COUNT(*)").From(tableName);
                    foreach (var existField in ExistColumns)
                    {
                        var existFieldValue = pd.Columns.Where(p => p.Key == existField).Select(p => p.Value.PropertyInfo.GetValue(poco, null)).Single();
                        existSql.Where(existField + "=@0", existFieldValue);
                    }

                    var existTotal = db.ExecuteScalar<long>(existSql);
                    if (existTotal > 0)
                        throw new RepositoryException("数据库已存在相同记录，请检查。");
                }

                db.Insert(tableName, primaryKeyName, true, poco);
            }
            else
            {
                if (!updateNullFields)
                {
                    IEnumerable<string> columns = pd.Columns.Where(p => p.Value.PropertyInfo.GetValue(poco, null) != null && p.Key.ToLower() != primaryKeyName.ToLower() && pd.QueryColumns.Contains(p.Key)).Select(p => p.Key).ToList<string>();
                    db.Update(tableName, primaryKeyName, poco, columns);
                }
                else {
                    db.Update(tableName, primaryKeyName, poco);
                }
            }
        }

        /// <summary>
        /// extjs分页
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="db"></param>
        /// <param name="page"></param>
        /// <param name="pageSize"></param>
        /// <param name="selectFileds"></param>
        /// <param name="filters"></param>
        /// <param name="sorts"></param>
        /// <returns></returns>
        public static Page<T> Page<T>(this Database db, long page, long pageSize, List<string> selectFileds = null, List<DataFilter> filters = null, List<DataSort> sorts = null,Sql sql = null)
        {
            string where = GetHqlstrByExtFilter(filters, "");
            string orderby = GetHqlstrByExtOrderBy(sorts, "");

            var pd = PocoData.ForType(typeof(T));
            string primaryKeyName = pd.TableInfo.PrimaryKey;

            if (sql == null) sql = Sql.Builder;
            if (!string.IsNullOrEmpty(where)) sql.Where(where);
            if (!string.IsNullOrEmpty(orderby)) sql.OrderBy(orderby.Replace("Id", primaryKeyName));

            return db.Page<T>(page, pageSize, sql);
        }

        /// <summary>
        /// 根据指定条件获取所有数据
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="db"></param>
        /// <param name="selectFileds"></param>
        /// <param name="filters"></param>
        /// <param name="sorts"></param>
        /// <param name="sql"></param>
        /// <returns></returns>
        public static List<T> GetAll<T>(this Database db, List<string> selectFileds = null, List<DataFilter> filters = null, List<DataSort> sorts = null, Sql sql = null)
        {
            string where = GetHqlstrByExtFilter(filters, "");
            string orderby = GetHqlstrByExtOrderBy(sorts, "");

            var pd = PocoData.ForType(typeof(T));
            string primaryKeyName = pd.TableInfo.PrimaryKey;

            if (sql == null) sql = Sql.Builder;
            if (!string.IsNullOrEmpty(where)) sql.Where(where);
            if (!string.IsNullOrEmpty(orderby)) sql.OrderBy(orderby.Replace("Id", primaryKeyName));

            return db.Query<T>(sql).ToList();
        }

        /// <summary>
        /// 处理extjs排序
        /// </summary>
        /// <param name="sorts"></param>
        /// <param name="a"></param>
        /// <returns></returns>
        public static string GetHqlstrByExtOrderBy(List<DataSort> sorts, string a)
        {
            if (sorts == null || sorts.Count == 0)
                return "";
            StringBuilder result = new StringBuilder();
            //type=string
            foreach (var i in sorts)
            {
                result.Append(a + i.property + " " + i.direction + " and ");
            }

            return result.ToString().Substring(0, result.Length - 4);
        }

        //处理ExtJs字段过滤
        public static string GetHqlstrByExtFilter(List<DataFilter> filters, string a)
        {
            if (filters == null || filters.Count == 0)
                return "1=1";
            StringBuilder result = new StringBuilder();
            //type=string
            var stringList = from f in filters where f.type == "string" select f;
            foreach (var i in stringList)
            {
                result.Append(a + i.field + " like '%" + i.value + "%' and ");
            }
            //type=stringlist
            var stringlistList = from f in filters where f.type == "stringlist" select f;
            foreach (var i in stringlistList)
            {
                string[] valuelist = i.value.Split(',');
                string inlist = "";
                for (int j = 0; j < valuelist.Count(); j++)
                {
                    if (inlist == "")
                    {
                        inlist = "'" + valuelist[j] + "'";
                    }
                    else
                    {
                        inlist = inlist + "," + "'" + valuelist[j] + "'";
                    }
                }
                result.Append(a + i.field + " in( " + inlist + ") and ");
            }
            //type=boolean
            var booleanList = from f in filters where f.type == "boolean" select f;
            foreach (var i in booleanList)
            {
                result.Append(a + i.field + "=" + i.value + " and ");
            }
            //type=numeric
            var numericList = from f in filters where f.type == "numeric" group f by f.field into g select g;
            foreach (var i in numericList)
            {
                result.Append("( ");
                string iiStr = string.Empty;
                foreach (var ii in i)
                {
                    iiStr += a + ii.field + GetComparison(ii.comparison) + ii.value + " and ";
                }
                result.Append(iiStr.Substring(0, iiStr.Length - 4));
                result.Append(" )");
                result.Append(" and ");
            }
            //type=date
            var dateList = from f in filters where f.type == "date" group f by f.field into g select g;
            foreach (var i in dateList)
            {
                result.Append("( ");
                string iiStr = string.Empty;
                foreach (var ii in i)
                {
                    iiStr += "DateDiff(dd,'" + ii.value + "'," + a + ii.field + ")" + GetComparison(ii.comparison) + " 0 " + " and ";
                }
                result.Append(iiStr.Substring(0, iiStr.Length - 4));
                result.Append(" )");
                result.Append(" and ");
            }
            //type=list  :"1,2"
            var listList = from f in filters where f.type == "list" select f;
            foreach (var i in listList)
            {
                if (i.value == "NULL")
                    result.Append(a + i.field + " is null and ");
                else
                    result.Append(a + i.field + " in ('" + i.value.Replace("\"", "'").Replace(",", "','") + "') and ");
            }

            return result.ToString().Substring(0, result.Length - 4);
        }

        private static string GetComparison(string comparison)
        {
            string res = string.Empty;
            switch (comparison)
            {
                case "lt":
                    res = "<";
                    break;
                case "gt":
                    res = ">";
                    break;
                case "eq":
                    res = "=";
                    break;
            }
            return res;
        }
    }

    /// <summary>
    /// 用来检测某字段在数据库中是否有相同记录
    /// </summary>
    [AttributeUsage(AttributeTargets.Property)]
    public class ExistColumnAttribute : Attribute
    {
    }
}
