﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using Nimmo.Framework.Common;
using Nimmo.Framework.Data.PetaPoco;
using Nimmo.Framework.ViewModel;
using Nimmo.Infrastructure.Relators;

namespace Nimmo.Infrastructure.Repositories
{
    public class UserInfoRepository : SingleRepository<UserInfo>
    {
        /// <summary>
        /// extjs分页
        /// </summary>
        /// <param name="page"></param>
        /// <param name="pageSize"></param>
        /// <param name="resultCount"></param>
        /// <param name="selectFileds"></param>
        /// <param name="filters"></param>
        /// <param name="sorts"></param>
        /// <returns></returns>
        public List<UserInfo> Paged(int TeamID, long page, long pageSize, out long resultCount, List<DataFilter> filters, List<DataSort> sorts, List<string> selectFileds = null)
        {
            Sql sql = Sql.Builder;
            sql.Append(string.Format("select {0} from (SELECT u.*,l.AddTime as LastLoginTime,l.IPAddress as LastLoginIp",selectFileds == null ? "*" : String.Join(",",selectFileds)));
            sql.Append("FROM Users u");
            sql.Append("LEFT JOIN OperationLogs l");
            sql.Append("ON OperationLogId=(select max(OperationLogId) from OperationLogs) and l.LogType='login' and l.UserID=u.UserId) v");
            if (TeamID > 0)
                sql.Where("TeamID=@0",TeamID);

            return this.Paged(page, pageSize, out resultCount, selectFileds, filters, sorts, sql);
        }

        /// <summary>
        /// 获取新注册用户
        /// </summary>
        /// <param name="dateType">时间类型：m本月、w本周、d今天</param>
        /// <returns></returns>
        public List<UserInfo> GetNewUsers(string dateType)
        {
            Sql sql = Sql.Builder;
            switch (dateType) { 
                case "m":
                    sql.Where("datediff(month,[AddTime],getdate())=0");
                    break;
                case "w":
                    sql.Where("datediff(week,[AddTime],getdate())=0");
                    break;
                case "d":
                    sql.Where("datediff(day,[AddTime],getdate())=0");
                    break;
            }

            return this.Query(sql).ToList();
        }

        /// <summary>
        /// 根据登录名判断用户是否存在
        /// </summary>
        /// <param name="LoginName"></param>
        /// <returns></returns>
        public bool ExistsByLoginName(string LoginName)
        {
            return this.Exists("LoginName=@0", LoginName);
        }
        /// <summary>
        /// 根据登录名和密码取得用户信息，如找不到相应用户则返回null
        /// </summary>
        /// <param name="LoginName"></param>
        /// <param name="LoginPassword"></param>
        /// <returns></returns>
        public UserInfo GetByLoginNameAndPwd(string LoginName, string LoginPassword)
        {
            Sql sql = Sql.Builder.Where("LoginName=@0 and LoginPwd=@1", LoginName, Utils.MD5(LoginName + LoginPassword));

            return this.Single(sql);
        }

        /// <summary>
        /// 根据登录名取得登录用户信息
        /// </summary>
        /// <param name="LoginName"></param>
        /// <returns></returns>
        public UserInfo GetByLoginName(string LoginName)
        {
            Sql sql = this.GetUserInfo()
                .Where("Users.LoginName=@0", LoginName);

            return this.db.Fetch<UserInfo,Team, Role, UserInfo>(
                new UserRelator().Map, sql).Single();
        }

        /// <summary>
        /// 密码输入是否正确
        /// </summary>
        /// <param name="UserID"></param>
        /// <param name="Pwd"></param>
        /// <returns></returns>
        public bool ExistUsersByPwd(int UserID, string Pwd)
        {
            return this.Exists("id=@0 and login_pwd=@1",UserID,Pwd);
        }

        /// <summary>
        /// 修改密码
        /// </summary>
        /// <param name="UserID"></param>
        /// <param name="Pwd"></param>
        public void UpdatePwd(int UserID, string Pwd)
        {
            this.Update("SET LoginPwd=@0 WHERE UserId=@1", Pwd, UserID);
        }

        /// <summary>
        /// 根据用户ID获取用户信息
        /// </summary>
        /// <param name="UserID"></param>
        /// <returns></returns>
        public UserInfo GetByUserID(int UserID)
        {
            Sql sql = this.GetUserInfo()
                .Where("Users.UserID=@0", UserID);

            return this.db.Fetch<UserInfo, Team, Role, UserInfo>(
                new UserRelator().Map, sql).Single();
        }

        private Sql GetUserInfo()
        {
            Sql sql = Sql.Builder.Select("*").From("Users")
                .Join("Teams").On("Teams.TeamID=Users.TeamID")
                .LeftOuterJoin("RolesToUsers").On("RolesToUsers.UserId=Users.UserId")
                .LeftOuterJoin("Roles").On("Roles.RoleId=RolesToUsers.RoleId");

            return sql;
        }

        public void SaveOrUpdateUserInfo(UserInfo entity)
        { 
            using (var trans = this.db.GetTransaction())
            {
                this.SaveOrUpdate(entity);
                if (entity.Roles.Count > 0)
                {
                    Sql sql = Sql.Builder;
                    sql.Append("delete from RolesToUsers where UserId=@0",entity.Id);
                    foreach (var item in entity.Roles)
                        sql.Append("insert into RolesToUsers (UserId,RoleId) values (@0,@1)", entity.Id, item.Id);

                    this.db.Execute(sql);
                }

                trans.Complete();
            }
        }
    }
}
